"use strict";
const path = require('path');
const IO = require('@merlinone-inc/storage-io/src/storage');

const Config = require('../config/service.config');

const workspace =  path.join(`${__dirname}/../workspace`);

/**
 * @typedef {import('moleculer').Context} Context Moleculer's Context
 */

module.exports = {
	name: "storage-io",
	version: 1,

	/**
	 * Settings
	 */
	settings: {

	},

	/**
	 * Dependencies
	 */
	dependencies: [],

	/**
	 * Actions
	 */
	actions: {

		/**
	 * Return a version string,  For testing to validate that the endpoint can be reached.
	 *
	 * @returns
	 */
	version: {
		rest: {
			method: "GET",
			path: "/version"
		},
		async handler() {
			return "0.0.1";
		}
	},


		/**
		 * Copy a file to a storage destination.  The source file will be a binary stream 'ctx.params' which will simply be forwarded to the underlying storage-io implementation.
		 * The site id of the customer that is making the request will be used to return the IO specific information regarding what storage-io implementation to use,
		 * as well as any login type parameters that must be passed to the underlying storage-io implementation.
		 *
		 * @param {String} siteid	- the identifier for the customer making the request.
		 * @param {String} destinationPath - the destination path where the file will be written.  If the path to the file does not exist it will be created.
		 * @param {bool} overwriteExisting  - if true the an existing file in the same path will be overwritten if it already exists.  (false by default)
		 */
		copy: {
			rest: "/copy",
			timeout: 10*60*1000, // 10 minnutes in milliseconds.  TODO:  revisit this.

/*			params: {
				siteid: "string",
				destinationPath: "string",
				overwriteExisting: "string",
			},*/
			/** @param {Context} ctx  */
			async handler(ctx) {
				// use a default of 4096 for the max file count in a directory
				const _kmaxFileCount = 4096;

				// TODO:  Retrieve the configuration info for the site making the request and use that info to initialize the storage-io implementation.
				// The number of files to store in each directory will be specified in the configuration.
				// At the root level of the Storage there will be a directory named after the site id of the customer.
				// Under each site id directory will be a set of directories with numeric names that each contain 'x' files.
				// It will be the responsibility of the underlying storage implementation to manage the creation of the directories.
				// For example, if we have a site id 'xyz', there will be directories as follows:
				// /xyz/000000/1001.jpg ... etc.
				// /xyz/000001/9901.jpg ... etc.
				//
				const maxFiles = _kmaxFileCount;		// TODO: read this from the site config settings.
				const opts = {
					siteid: ctx.meta.$params.siteid,
					destFile: ctx.meta.$params.destinationPath,
					sourceFile: ctx.params,
					overwrite: ctx.meta.$params.overwriteExisting,
					basePath: `${workspace}/${ctx.meta.$params.siteid}`,			// TODO: get the base path from the site config info.  For now simply use the siteid name.
					maxFiles: maxFiles,
				};
				const io = Config.IOFactory(opts);
				await io.copyFile(opts);
			}
		}
	},

	/**
	 * Events
	 */
	events: {

	},

	/**
	 * Methods
	 */
	methods: {

	},

	/**
	 * Service created lifecycle event handler
	 */
	created() {

	},

	/**
	 * Service started lifecycle event handler
	 */
	async started() {

	},

	/**
	 * Service stopped lifecycle event handler
	 */
	async stopped() {

	}
};
